--[[
Broker_SimpleCurrency - simple currency data feed

Copyright (C) 2008-2009 Adirelle

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
--]]

local ExpandCurrencyList = _G.ExpandCurrencyList
local GetCurrencyListInfo = _G.GetCurrencyListInfo
local GetCurrencyListSize = _G.GetCurrencyListSize
local tinsert = _G.tinsert
local tremove = _G.tremove
local wipe = _G.wipe

local eventFrame = CreateFrame("Frame")
local dataobj = LibStub('LibDataBroker-1.1'):NewDataObject('SimpleCurrency', { 
	type = 'data source',
	icon = [[Interface\MoneyFrame\UI-GoldIcon]],
})

local ICON_STRING = "\124T%s:0:0:2:0\124t"
local HONOR_ICON_STRING
local ARENA_ICON_STRING = ICON_STRING:format([[Interface\PVPFrame\PVP-ArenaPoints-Icon]])

local MONEY_FORMAT_STRING = GOLD_AMOUNT_TEXTURE .. ' ' .. SILVER_AMOUNT_TEXTURE .. ' ' .. COPPER_AMOUNT_TEXTURE

local numData = 0
local currencyData = {}

local function NewData()
	numData = numData + 1 
	local data = currencyData[numData]
	if not data then
		data = {}
		currencyData[numData] = data
	end
	return data	
end

function dataobj.OnTooltipShow(tooltip)
	tooltip = tooltip or GameTooltip
	tooltip:ClearLines()
	tooltip:AddLine(CURRENCY, 1, 1, 1)
	for index,data in ipairs(currencyData) do
		if data.value then
			tooltip:AddDoubleLine(data.label, data.value)
		else
			tooltip:AddLine(data.label, 1, 1, 1)
		end
	end
end

function dataobj.OnClick()
	ToggleCharacter("TokenFrame")
end

local headerState = {}
local shownCurrencies = {}
local function UpdateData()
	if InCombatLockdown() then
		return
	end

	-- Clean up
	wipe(shownCurrencies)
	numData = 0
	
	-- Update money data
	local money = GetMoney()
	local coppers = money % 100
	local silvers = math.floor(money / 100) % 100
	local golds = math.floor(money / 10000)	
	local data = NewData()
	data.label = MONEY
	data.value = MONEY_FORMAT_STRING:format(golds, 0, 0, silvers, 0, 0, coppers, 0, 0)
	
	local moneyString = ""
	if money >= 10000 then 
		moneyString = GOLD_AMOUNT_TEXTURE:format(golds, 0, 0)
	end
	if money >= 100 and money < 100000 then
		moneyString = moneyString .. " " .. SILVER_AMOUNT_TEXTURE:format(silvers, 0, 0)
	end
	if money < 1000 then
		moneyString = moneyString .. " " .. COPPER_AMOUNT_TEXTURE:format(coppers, 0, 0)
	end
	tinsert(shownCurrencies, moneyString:trim())
	
	-- Update token data
	local wasHeader = false
	local index = 1
	while index <= GetCurrencyListSize() do
		local name, isHeader, isExpanded, isUnused, isWatched, count, extraCurrencyType, icon = GetCurrencyListInfo(index)
		if not isUnused then
			if not isHeader or not wasHeader then
				data = NewData()
			-- else reuse previous data table
			end
			data.label = name
			if isHeader then
				data.value = nil		
				if not isExpanded then
					-- Expand all headers
					headerState[index] = true
					ExpandCurrencyList(index, 1)
				end
			else
				local iconString
				if extraCurrencyType == 1 then	
					-- Arena points
					iconString = ARENA_ICON_STRING			
				elseif extraCurrencyType == 2 then 
					-- Honor points
					if HONOR_ICON_STRING == nil then
						local factionGroup = UnitFactionGroup("player")
						if factionGroup then
							HONOR_ICON_STRING = ICON_STRING:format([[Interface\AddOns\Broker_SimpleCurrency\images\]]..factionGroup)
						end
					end
					iconString = HONOR_ICON_STRING or " "..HONOR
				else
					-- "Standard" token
					iconString = icon and ICON_STRING:format(icon) or ""
				end
				data.value = ("%d%s"):format(count, iconString)
				if isWatched then
					-- Display it directly in the LDB if asked for
					tinsert(shownCurrencies, data.value)
				end
			end
			wasHeader = isHeader
		end
		index = index + 1
	end
	
	-- Discard remaining tables
	for i = numData + 1, #currencyData do
		currencyData[i] = nil
	end
	
	-- Update broker text
	dataobj.text = table.concat(shownCurrencies, "  ")

	-- Restore header expanded state
	for index in pairs(headerState) do
		ExpandCurrencyList(index, 0)
		headerState[index] = nil
	end
end

eventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")

eventFrame:SetScript('OnEvent', function(self)
	self:UnregisterEvent('PLAYER_ENTERING_WORLD')
	
	UpdateData()

	self:RegisterEvent("MERCHANT_CLOSED")
	self:RegisterEvent("PLAYER_TRADE_MONEY")
	self:RegisterEvent("TRADE_MONEY_CHANGED")
	self:RegisterEvent("SEND_MAIL_MONEY_CHANGED")
	self:RegisterEvent("SEND_MAIL_COD_CHANGED")
	self:RegisterEvent('HONOR_CURRENCY_UPDATE')
	self:RegisterEvent('PLAYER_MONEY')
	self:RegisterEvent('BAG_UPDATE')
	self:RegisterEvent('PLAYER_REGEN_ENABLED')

	if type(TokenFrame_Update) == "function" then
		hooksecurefunc('TokenFrame_Update', UpdateData)
		self:SetScript('OnEvent', UpdateData)
	else
		self:RegisterEvent('ADDON_LOADED')
		self:SetScript('OnEvent', function(self, event, arg1, ...)
			if event == 'ADDON_LOADED' then
				if arg1 == 'Blizzard_TokenUI' then
					print(self, event, arg1, ...)
					hooksecurefunc('TokenFrame_Update', UpdateData)
					self:UnregisterEvent('ADDON_LOADED')
					self:SetScript('OnEvent', UpdateData)
				end
				return
			end
			UpdateData()
		end)
	end
end)

